#!/usr/bin/perl -w
# $Id: exchange.pl,v 1.3 2005/09/02 21:55:45 oliver Exp $

use strict;
use Getopt::Long;
use File::Basename;
use File::Temp qw(tempfile tempdir);
use FileHandle;
use lib "/sansom/gfio/oliver/Devel/Setup/UmbrellaSampling";
use Messages qw($DEBUG open_log errmsg msg logger warning date Progname);
use Data::Dumper;

my $program_version = q( $Id: exchange.pl,v 1.3 2005/09/02 21:55:45 oliver Exp $ );
my $progname = Progname();
my $gmx_version = "3.2.1-lmb";
my ($XX,$YY,$ZZ) = (0,1,2);           # use names for dimensions in arrays

my $TOPOLOGY = "topol.top";
my $CONFSTART = "conf.dat";
my $MDP = "grompp.mdp";

my $PROJECT = ".";

my $TYPE;                 # particle type (name from gro file)
my ($Z1,$Z2);             # only exchange TYPE wit z<Z1 or z>Z2

my ($show_help,$show_targets,$show_config,$show_version,$opt_verbose);

# "--topology"  => \$TOPOLOGY, 
#                      "--mdp"       => \$MDP,
my %requiredfiles = ("--confstart" => \$CONFSTART, 
                     );
my %requiredvars = ( "--type"      => \$TYPE,
                     "--z1"        => \$Z1,
                     "--z2"        => \$Z2,
                     );

my %UmbrellaParams;   # stores global parameters set through the configuration list
my %configvars = (    # allowed vars in UmbrellaParams ('required' will be changed to
                   );

# Application programs used while running prepumbrella.pl
my %Application = ( "grompp" => "grompp",
                "GMXRC"  => "source /sansom/fedpacks/opt/gromacs/3.2.1/i686-pc-linux-gnu/bin/GMXRC",
                "editconf" => "/sansom/fedpacks/opt/gromacs/3.2.1/i686-pc-linux-gnu/bin/editconf",
                "vmd" => "/sansom/fedpacks/bin/vmd",
                );


my $convfac;              # holds unit conversion factors


my $startconfigs;
# [i]->{filename}->"file"
#    ->{position}->[x,y,z]
#    ->{atom_id}->atom_id
#    ->{type}->"string"     type = Na|Cl|SOL|...  



&parseArgs();
if ($show_help) {
    &print_help;
    exit(0);
} elsif ($show_version) {
    print "version: ".$program_version . "\n";
    exit(0);
} elsif ($show_config) {
    &print_configformat;
    exit(0);
};
&open_log;
&checkArgs;

$startconfigs = &parseConfigList($CONFSTART);  # also checks that com is supplied

my ($c,$tmpfh,$tmpfn);
my ($oldfn, $oldpath, $oldsuffix, $newfn, $newpdb,$newgro);
my ($cmd);
foreach $c (@$startconfigs) {
    ($tmpfh,$tmpfn) = tempfile("exchangeXXXXXX",SUFFIX=>".vmd",UNLINK=>1);
    ($oldfn, $oldpath, $oldsuffix) = fileparse($c->{filename},(".gro",".pdb"));
    $newfn = $oldfn."x".$c->{atom_id};
    $newpdb = $oldpath.$newfn.".pdb";
    $newgro = $oldpath.$newfn.".gro";
    msg(3,"oldfn: $c->{filename}; atom_id: $c->{atom_id}\n");
    msg(3,"newfn: $newfn; path: $oldpath;  vmdtmp: $tmpfn\n");

    &createVMDcommands($tmpfh,$c,$newpdb);
    $cmd = "$Application{vmd} -dispdev text -e $tmpfn";
    msg(5,"Executing: $cmd\n");
    system($cmd) == 0 or  die "Error executing vmd: $?";
    msg(3,"vmd exchanged water and ion");

    $cmd = "$Application{editconf} -f ${newpdb} -o ${newgro} >/dev/null 2>&1";
    msg(5,"Executing: $cmd\n");
    system($cmd) == 0 or  die "Error coverting ${newpdb} -> ${newgro}: $?";
    msg(1,"Created $newgro (from $newpdb)");
    
    msg(5,"Removing tmp file $newpdb\n");
    unlink $newpdb;
}


exit(0);
# end main

sub createVMDcommands ($$) {
    my ($fh, $c,$pdb) = @_;
# perl is crap at 'here' documents: it is impossible to print '$name' AND
# use interpolation.
# Hence pack everything in an array and spit it out line by line
    my @template = (
                    q(
#------------------------------------------------------------------
# exchange an ion from the edge of the box with a particular water
#------------------------------------------------------------------
                      ),
                    qq(
mol addfile "$c->{filename}"
set owindex  $c->{atom_id}
set pdbfile "$pdb"),
                    q(
set owindex [expr $owindex - 1]),
                    qq(
set ion [atomselect top "resname '$TYPE' and (z<$Z1 or z>$Z2)"]),
                    q(
# (just assume that we have found at least one ion...)
puts "Found [llength [$ion list]] ions; taking the first one, index:"
set ion_index [lindex [$ion get index] 0]
set ion [atomselect top "index $ion_index"] 
puts "Ion: resname [$ion get resname], resid [$ion get resid]"
puts "Original position:"
set ion_pos [lindex [$ion get {x y z}] 0]

# water molecule from OW
set ow [atomselect top "index $owindex"]
puts "Water index=$owindex from input file, resname [$ow get resname], resid [$ow get resid]"
puts "Original position:"
set ow_pos [lindex [$ow get {x y z}] 0]
# now get the whole water molecule
set water [atomselect top "same residue as index $owindex"]

# shift vector
puts "Shift by dx = x(ion) - x(water) = "
set dx [vecsub $ion_pos $ow_pos]

$water moveby $dx
$ion moveby [vecscale -1 $dx]

# write pdb (loosing velocities)
set system [atomselect top all]
$system writepdb "$pdbfile"

puts "VMD: Wrote pdb '$pdbfile' with exchanged particles"
quit)
                    ); # end template
    foreach (@template) {
        print $fh $_."\n";
    }    
}



sub parseArgs () {
    &GetOptions ("help"         => \$show_help,
                 "help-config"  => \$show_config,
                 "version"      => \$show_version,
                 "verbose"      => \$opt_verbose,
                 "debug=i"      => \$DEBUG,
                 "type=s"       => \$TYPE,
                 "z1=f"         => \$Z1,
                 "z2=f"         => \$Z2,
                 "confstart=s"  => \$CONFSTART,
                 "topology=s"   => \$TOPOLOGY,
                 "mdp=s"        => \$MDP,
                 "Project=s"    => \$PROJECT,
                 ) or do {
                     &print_help();
                     exit(1);
                 };
    $DEBUG = 3 if ($opt_verbose && !$DEBUG);
};

sub print_help () {
    print << "EHELP";
usage: $progname [options]

Exchange ions of given type with a water molecule. The water molecules
are given in list (conf.list, same as for prepumbrella.pl and which is
typically produced from output of prepconflist.pl)

Options (defaults in brackets):

--help                  this help
--help-config           format of the starting configuration file (for
                        --confstart)
--version               version number of $progname (quote when 
                        submitting a bug report)
--verbose               chatty output
--debug=INTEGER         set debug level (>9 switches off error checks, >11 
                        grompp is not run at all, only commandline shown)

--type=STRING           name of the particle, eg Na, Cl, NA, ... SOL (as used
                        in the input files)
--z1=FLOAT              only exchange with TYPE molecules whose box z coordinate
--z2=FLOAT              is z<Z1 or z>Z2 (ie at the edge of the box) [nm]

Input files (all must exist):
#--topology=FILE         gromacs topology [$TOPOLOGY]
#--mdp=FILE              grompp input with run parameters  [$MDP]
--confstart=FILE        list that describes available starting conformations
                        (see below for format) [$CONFSTART]

BUGS:
====
* velocities are lost (pdb -> gro), therefore 
      gen_velocities = YES
  in the mdp file
* no sanity checks, be careful with Z1 and Z2
* Z1 and Z2 could be given relative to the box edge but that would require
  more vmd tcl coding to find the box dimensions... uargh
* perhaps should energy minimise the exchanged structures?
EHELP
};

sub print_configformat {
    print << "ECONFIGFORMAT";
=======================================================================
Format of the starting configuration file


----------------------------------------------------------------------
; comment (same as in Gromacs mdp files)
; all entries must be on one line
; '\@' lines set variables 
\@ var = value
filename   particle_type  atom_nr   pos_x pos_y pos_z 
filename   particle_type  atom_nr   pos_x pos_y pos_z
...
----------------------------------------------------------------------


filename       ::= full path to the gro or pdb file
particle_type  ::= name used in Gromacs, eg Na, Cl, OW, SOL
atom_nr        ::= atom_id, number in gro or index file
pos_X|Y|Z      ::= coordinates of the particle


optional variables (none are used at the moment):
center_of_mass = X Y Z      ; center of mass of the reference group
reference_group = group     ; reference group 
reftype = com_t0|com        ; how to determine the com
RConfine = radius           ; nm
KConfine = forceconst       ; kJ/(mol*nm^2)

You can use prepconflist.pl (see 
<https://sansom.biop.ox.ac.uk/cgi-bin/cvsweb/Gromacs/Setup/UmbrellaSampling/prepconflist.pl>) for setting up the configuration list from trr frames.

==========================================================================
ECONFIGFORMAT
};



sub checkArgs {
    my $argerror = 0;
    my $opt;


    foreach $opt (keys %requiredfiles) {
        my $file = ${$requiredfiles{$opt}};
        next if (-e $file);
        $argerror = 2;
        errmsg("Required file `%s', set with option %s,  not found.\n",
               $file,$opt);
    };

    foreach $opt (keys %requiredvars) {
        my $var = ${$requiredvars{$opt}};
        next if ($var);
        $argerror = 3;
        errmsg("Required parameter associated with option %s was not set.\n",
               $opt);
    }

    if ($argerror && $DEBUG < 10) {
        errmsg(" EXIT. Type $progname --help for the correct syntax.\n");
        exit($argerror);
    };

    # need Z1 and Z2 in Angstrom
    $Z1 *= 10.0;  $Z2 *= 10.0;
}

sub parseConfigList ($) {
    # better use XML::Simple
    my ($cfglist) = @_;
    my (@c,$filename,$type,$atom_id,$x,$y,$z,@comment);
    my ($key,$val);
    my $linenr = 0;
    
    open(CFG,"< $cfglist") or die "Cannot open $cfglist, stopped";
    while (<CFG>) {
        chomp;
        ++$linenr;
        next if (/^\s*;/ || /^\s*$/);      # skip comment and empty lines
        s(\s*;.*$)();                      # kill trailing whitespace and  comments
        if (/^\s*@/) {                     # We want to set a  variable...
            s(^\s*@\s*)();                 # kill @ indicator
            ($key,$val) = split /\s*=\s*/; # key = value pairs
            msg(11,"parseConfigList: key='%s' val='%s'\n",$key,$val);

            if ($configvars{$key}) {       # exists as allowed key word
                msg(10,"parseConfigList: \`$key' is a valid variable name.\n");
                $configvars{$key}->{"isset"} = 1;      # mark as read    
                if ($configvars{$key}->{"dim"} > 1) {  # array is expected
                    $UmbrellaParams{$key} = [split / /,$val];
                } else {
                    $UmbrellaParams{$key} = $val;
                }
            } else {
                warning("parseConfigList: variable \`%s' at line %i is not "
                        ."recognised (and ignored)\n",$key,$linenr);
            }
            next;
        };

        # We want to define a  configuration entry:
        ($filename,$type,$atom_id,$x,$y,$z,@comment) = split;

        # should do sanity check here but we will trust the input for now...
        push @c, { "filename" => $filename, 
                   "type" => $type, 
                   "atom_id" => $atom_id, 
                   "position" => [$x,$y,$z] };
    }
    close(CFG);


    # now check that all variables are set
    my $die_later = 0;
    foreach $key (keys %configvars) {
        if (!$configvars{$key}->{"isset"}) {
            if ($configvars{$key}->{"required"}) {
                $die_later = 1;
                errmsg("The variable \`$key' MUST be set in the configuration "
                       ."file $cfglist... this is fatal.\n");
            } else {
                # set default
                warning("parseConfigList: %s defaulted to \`%s'.\n",
                        $key,$configvars{$key}->{"default"});
                $UmbrellaParams{$key} = $configvars{$key}->{"default"};
            }
        }
    }
        
    msg(11, "parseConfigList: UmbrellaParams = %s\n",Dumper(\%UmbrellaParams));
    msg(11, "parseConfigList: startconfigs = %s\n",Dumper(\@c));
    die "Error in the configuration list $cfglist, stopped" if ($die_later);        
    return \@c;
}



