#!/bin/bash
#$ -N pmftolscan
#$ -S /bin/bash
#$ -cwd
#$ -j y
#$ -r y

# $Id: tolscan.sh,v 1.5 2004/07/05 16:48:48 oliver Exp $
# runscript for SGE to analyse Umbrella output (pdo) files from mdrun 
# (or simulated output from fakepmf) for different levels of the convergence 
# threshold
#
# MIN and MAX of the histogram are determined automatically but can be faked by 
# creating 'limits.dat' which contains two lines (use scientific notation!):
# --------------------------
# Final min: -6.78e-01
# Final max: 1.5554e+01
# --------------------------

TEMPERATURE=298.0
BINS=250

T_BEGIN=-1
T_END=-1

WINDOWS_D=.
# TOLERANCES="1e-1 1e-2 1e-3 1e-4 1e-5 1e-6 1e-7 1e-8"
TOLERANCES="1e-3 1e-4 1e-5 1e-6 1e-7"

find_pdo="find ${WINDOWS_D} -name '*.pdo.gz'"
view_cmd="xmgrace"

echo "Parameters:"
echo "TEMPERATURE=${TEMPERATURE} K,  BINS=${BINS}"
echo "T_BEGIN=${T_BEGIN} ps, T_END=${T_END} ps"

source /sansom/fedpacks/opt/gromacs/3.2.1/i686-pc-linux-gnu/bin/GMXRC

if [ ! -e limits.dat ]; then
    # clean up on first run (no need to check RESTART == 1)
    rm -f 1e-[1-9].xvg

    echo "Finding min and max of the histogram (first pass, no WHAM)"
    cmd="${find_pdo} | xargs g_wham -bins ${BINS} -b ${T_BEGIN} -e ${T_END}  -noprof"
    eval ${cmd} | grep "^Final m" > limits.dat
fi

MIN=$(sed -n -e '/min/s/^.*:[[:space:]]*\([-+.0-9]*[eE][-+][0-9]\+\)/\1/p' limits.dat)
MAX=$(sed -n -e '/max/s/^.*:[[:space:]]*\([-+.0-9]*[eE][-+][0-9]\+\)/\1/p' limits.dat)

echo "Using limits MIN=${MIN} and MAX=${MAX}"

echo "Unbiasing (WHAM) and PMF for decreasing convergence thresholds ${TOLERANCES}  kT"
nset=0
for tol in ${TOLERANCES}; do 
    echo "[ tol = $tol ]"; 
    profile=${tol}.xvg
    cmd="${find_pdo} | \
           xargs g_wham -bins ${BINS} -temp ${TEMPERATURE} -b ${T_BEGIN} -e ${T_END}\
                -tol ${tol}  -min ${MIN} -max ${MAX} -o ${profile}"
    eval ${cmd}

    # remove infs
    echo "Removing 'inf' from output ${profile}"
    cp ${profile} tmp.xvg
    grep -v inf tmp.xvg > ${profile}
    rm tmp.xvg

    view_cmd="${view_cmd} -block ${profile} -bxy 1:3 -pexec 's${nset} legend \"${tol}\"'"
    let nset++
done
rm -f \#*


echo "Display PMF by executing ./show_pmf.sh"

echo -e '#!/bin/sh\n'"${view_cmd}\n" > show_pmf.sh
chmod a+x show_pmf.sh
